@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import os
 import sys
 
@@ -13,7 +9,7 @@ from . import util
 # TODO: Some verification of the config values
 
 
-class Config(object):
+class Config:
     """
     Manages the configuration for a benchmark project.
     """
@@ -25,7 +21,7 @@ class Config(object):
         self.repo = None
         self.repo_subdir = ""
         self.branches = [None]
-        self.pythons = ["{0[0]}.{0[1]}".format(sys.version_info)]
+        self.pythons = [f"{sys.version_info[0]}.{sys.version_info[1]}"]
         self.matrix = {}
         self.exclude = []
         self.include = []
@@ -36,7 +32,8 @@ class Config(object):
         self.show_commit_url = "#"
         self.hash_length = 8
         self.environment_type = None
-        self.install_timeout = 600
+        self.install_timeout = 600.0
+        self.default_benchmark_timeout = 60.0
         self.dvcs = None
         self.regressions_first_commits = {}
         self.regressions_thresholds = {}
@@ -57,19 +54,20 @@ class Config(object):
             path = "asv.conf.json"
 
         if not os.path.isfile(path):
-            raise util.UserError("Config file {0} not found.".format(path))
+            raise util.UserError(f"Config file {path} not found.")
 
         d = util.load_json(path, cls.api_version, js_comments=True)
         try:
             return cls.from_json(d)
         except ValueError:
             raise util.UserError(
-                "No repo specified in {0} config file.".format(path))
+                f"No repo specified in {path} config file.")
 
     @classmethod
     def from_json(cls, d):
         if 'wheel_cache_size' in d:
-            log.warning("`wheel_cache_size` has been renamed to `build_cache_size`. Update your `asv.conf.json` accordingly.")
+            log.warning("`wheel_cache_size` has been renamed to `build_cache_size`."
+                        " Update your `asv.conf.json` accordingly.")
             d.setdefault('build_cache_size', d['wheel_cache_size'])
 
         conf = cls()
