@@ -15,6 +15,8 @@ import subprocess
 import importlib
 from pathlib import Path
 
+import tomli
+
 from .console import log
 from . import util, build_cache
 
@@ -440,8 +442,11 @@ def get_environment_class_by_name(environment_type):
     for cls in util.iter_subclasses(Environment):
         if cls.tool_name == environment_type:
             return cls
+    tool_names = [cls.tool_name for cls in util.iter_subclasses(Environment)]
     raise EnvironmentUnavailable(
-        f"Unknown environment type '{environment_type}'")
+        f"Unknown environment type '{environment_type}'. "
+        f"Allowed values based on existing plugins are {tool_names}."
+    )
 
 
 def is_existing_only(environments):
@@ -509,6 +514,8 @@ class Environment:
         # These are needed for asv to build and run the project, not part of
         # benchmark name mangling
         self._base_requirements = {}
+        # gh-1385
+        self._base_requirements["pip+build"] = ""
         # gh-1314
         asv_runner_path = os.getenv("ASV_RUNNER_PATH", "")
         module_path = Path(asv_runner_path) / "asv_runner"
@@ -534,11 +541,14 @@ class Environment:
             # XXX: What if pypy installed asv tries to benchmark a cpython
             # python?
             self._base_requirements["pip+pympler"] = ""
-        if (Path.cwd() / "poetry.lock").exists():
-            self._base_requirements["poetry-core"] = ""
 
-        if (Path.cwd() / "pdm.lock").exists():
-            self._base_requirements["pdm"] = ""
+        pyproject_path = Path.cwd() / "pyproject.toml"
+        if pyproject_path.exists():
+            with open(pyproject_path, "rb") as pyproject_file:
+                pyproject_data = tomli.load(pyproject_file)
+                requires = pyproject_data.get("build-system", {}).get("requires", [])
+                for requirement in requires:
+                    self._base_requirements[f"pip+{requirement}"] = ""
 
         # Update the _base_requirements if needed
         for key in list(self._requirements.keys()):
@@ -907,26 +917,14 @@ class Environment:
         """
         Run build commands
         """
-        build_dir_path = Path(build_dir)
-
-        def has_file(file_name):
-            return (build_dir_path / file_name).exists()
 
         cmd = self._build_command
 
         if cmd is None:
-            if has_file('pyproject.toml'):
-                cmd = [
-                    ("PIP_NO_BUILD_ISOLATION=false "
-                     "python -mpip wheel --no-deps --no-index "
-                     "-w {build_cache_dir} {build_dir}")
-                ]
-            else:
-                cmd = [
-                    "python setup.py build",
-                    ("PIP_NO_BUILD_ISOLATION=false "
-                     "python -mpip wheel --no-deps --no-index -w {build_cache_dir} {build_dir}")
-                ]
+            cmd = [
+                "PIP_NO_BUILD_ISOLATION=0 python -m build",
+                "python -mpip wheel -w {build_cache_dir} {build_dir}"
+            ]
 
         if cmd:
             commit_name = repo.get_decorated_hash(commit_hash, 8)
