@@ -1,12 +1,9 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 """
 Various low-level utilities.
 """
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 import datetime
 import json
@@ -16,7 +13,6 @@ import re
 import select
 import signal
 import subprocess
-import struct
 import sys
 import time
 import errno
@@ -27,27 +23,19 @@ import shlex
 import operator
 import collections
 import multiprocessing
+import functools
 
-import six
-from six.moves import xrange
-
-from .extern import minify_json
-
-
-nan = float('nan')
-inf = float('inf')
+import json5
+from asv_runner.util import human_time, human_float, _human_time_units
 
 WIN = (os.name == 'nt')
 
 if not WIN:
-    try:
-        from select import PIPE_BUF
-    except ImportError:
-        # PIPE_BUF is not available on Python 2.6
-        PIPE_BUF = os.pathconf('.', os.pathconf_names['PC_PIPE_BUF'])
+    from select import PIPE_BUF
 
 
 TIMEOUT_RETCODE = -256
+terminal_width = shutil.get_terminal_size().columns
 
 
 class UserError(Exception):
@@ -81,72 +69,20 @@ class ParallelFailure(Exception):
             raise self
 
 
-def human_list(l):
+def human_list(input_list):
     """
     Formats a list of strings in a human-friendly way.
     """
-    l = ["'{0}'".format(x) for x in l]
+    input_list = [f"'{x}'" for x in input_list]
 
-    if len(l) == 0:
+    if len(input_list) == 0:
         return 'nothing'
-    elif len(l) == 1:
-        return l[0]
-    elif len(l) == 2:
-        return ' and '.join(l)
+    elif len(input_list) == 1:
+        return input_list[0]
+    elif len(input_list) == 2:
+        return ' and '.join(input_list)
     else:
-        return ', '.join(l[:-1]) + ' and ' + l[-1]
-
-
-def human_float(value, significant=3, truncate_small=None, significant_zeros=False):
-    """
-    Return a string representing a float with human friendly significant digits.
-    Switches to scientific notation for too large/small numbers.
-    If `truncate_small`, then leading zeros of numbers < 1 are counted as 
-    significant. If not `significant_zeros`, trailing unnecessary zeros are 
-    stripped.
-    """
-    if value == 0:
-        return "0"
-    elif math.isinf(value) or math.isnan(value):
-        return "{}".format(value)
-    elif value < 0:
-        sign = "-"
-        value = -value
-    else:
-        sign = ""
-
-    logv = math.log10(value)
-    magnitude = int(math.floor(logv)) + 1
-
-    if truncate_small is not None:
-        magnitude = max(magnitude, -truncate_small + 1)
-
-    num_digits = significant - magnitude
-
-    if magnitude <= -5 or magnitude >= 9:
-        # Too many digits, use scientific notation
-        fmt = "{{0:.{0}e}}".format(significant)
-    elif value == int(value):
-        value = int(round(value, num_digits))
-        fmt = "{0:d}"
-    elif num_digits <= 0:
-        value = int(round(value, num_digits))
-        fmt = "{0:d}"
-    else:
-        fmt = "{{0:.{0}f}}".format(num_digits)
-
-    formatted = sign + fmt.format(value)
-
-    if not significant_zeros and '.' in formatted and 'e' not in fmt:
-        formatted = formatted.rstrip('0')
-        if formatted[-1] == '.':
-            formatted = formatted[:-1]
-
-    if significant_zeros and '.' not in formatted:
-        if len(formatted) < significant:
-            formatted += "." + "0"*(significant - len(formatted))
-
-    return formatted
+        return ', '.join(input_list[:-1]) + ' and ' + input_list[-1]
 
 
 def human_file_size(size, err=None):
@@ -191,77 +127,10 @@ def human_file_size(size, err=None):
     str_value = human_float(value, 3)
 
     if err is None:
-        return "{0:s}{1}".format(str_value, suffix)
+        return f"{str_value:s}{suffix}"
     else:
         str_err = human_float(err / scale, 1, truncate_small=2)
-        return "{0:s}±{1:s}{2}".format(str_value, str_err, suffix)
-
-
-_human_time_units = (
-    ('ns', 0.000000001),
-    ('μs', 0.000001),
-    ('ms', 0.001),
-    ('s', 1),
-    ('m', 60),
-    ('h', 60 * 60),
-    ('d', 60 * 60 * 24),
-    ('w', 60 * 60 * 24 * 7),
-    ('y', 60 * 60 * 24 * 7 * 52),
-    ('C', 60 * 60 * 24 * 7 * 52 * 100)
-)
-
-
-def human_time(seconds, err=None):
-    """
-    Returns a human-friendly time string that is always exactly 6
-    characters long.
-
-    Depending on the number of seconds given, can be one of::
-
-        1w 3d
-        2d 4h
-        1h 5m
-        1m 4s
-          15s
-
-    Will be in color if console coloring is turned on.
-
-    Parameters
-    ----------
-    seconds : int
-        The number of seconds to represent
-
-    Returns
-    -------
-    time : str
-        A human-friendly representation of the given number of seconds
-        that is always exactly 6 characters.
-    """
-    units = _human_time_units
-    seconds = float(seconds)
-
-    scale = seconds
-
-    if scale == 0 and err is not None:
-        scale = float(err)
-
-    if scale == 0:
-        # Represent zero in reasonable units
-        units = [('s', 1), ('m', 60)]
-
-    if scale != scale:
-        # nan
-        return "n/a"
-
-    for i in xrange(len(units) - 1):
-        if scale < units[i+1][1]:
-            str_time = human_float(seconds / units[i][1], 3, significant_zeros=True)
-            if err is None:
-                return "{0:s}{1}".format(str_time, units[i][0])
-            else:
-                str_err = human_float(err / units[i][1], 1, truncate_small=2)
-                return "{0:s}±{1:s}{2}".format(str_time, str_err, units[i][0])
-    return '~0'
+        return f"{str_value:s}±{str_err:s}{suffix}"
 
 
 def human_value(value, unit, err=None):
@@ -290,7 +159,7 @@ def human_value(value, unit, err=None):
         else:
             display = json.dumps(value)
             if err is not None:
-                display += "±{:.2g}".format(err)
+                display += f"±{err:.2g}"
     elif value is None:
         display = "failed"
     else:
@@ -325,10 +194,10 @@ def which(filename, paths=None):
     """
     Emulates the UNIX `which` command in Python.
 
-    Raises an IOError if no result is found.
+    Raises an OSError if no result is found.
     """
     # Hide traceback from expected exceptions in pytest reports
-    __tracebackhide__ = operator.methodcaller('errisinstance', IOError)
+    __tracebackhide__ = operator.methodcaller('errisinstance', OSError)
 
     if os.path.sep in filename:
         locations = ['']
@@ -358,7 +227,7 @@ def which(filename, paths=None):
             loc_info = 'PATH'
         else:
             loc_info = os.pathsep.join(locations)
-        raise IOError("Could not find '{0}' in {1}".format(filename, loc_info))
+        raise OSError(f"Could not find '{filename}' in {loc_info}")
 
     return candidates[0]
 
@@ -369,7 +238,7 @@ def has_command(filename):
     """
     try:
         which(filename)
-    except IOError:
+    except OSError:
         return False
     else:
         return True
@@ -384,8 +253,7 @@ class ProcessError(subprocess.CalledProcessError):
 
     def __str__(self):
         if self.retcode == TIMEOUT_RETCODE:
-            return "Command '{0}' timed out".format(
-                ' '.join(self.args))
+            return f"Command '{' '.join(self.args)}' timed out"
         else:
             return "Command '{0}' returned non-zero exit status {1}".format(
                 ' '.join(self.args), self.retcode)
@@ -408,7 +276,7 @@ def check_call(args, valid_return_codes=(0,), timeout=600, dots=True,
         cwd=cwd)
 
 
-class DebugLogBuffer(object):
+class DebugLogBuffer:
     def __init__(self, log):
         self.buf = []
         self.first = True
@@ -523,16 +391,10 @@ def check_output(args, valid_return_codes=(0,), timeout=600, dots=True,
             ])
         return '\n'.join(content)
 
-    if isinstance(args, six.string_types):
+    if isinstance(args, str):
         args = [args]
 
-    log.debug("Running '{0}'".format(' '.join(args)))
-
-    if env and WIN and sys.version_info < (3,):
-        # Environment keys and values cannot be unicode
-        def _fix_env(s):
-            return s.encode('mbcs') if isinstance(s, unicode) else s
-        env = {_fix_env(k): _fix_env(v) for k, v in env.items()}
+    log.debug(f"Running '{' '.join(args)}'")
 
     kwargs = dict(shell=shell, env=env, cwd=cwd,
                   stdout=subprocess.PIPE, stderr=subprocess.PIPE)
@@ -566,7 +428,8 @@ def check_output(args, valid_return_codes=(0,), timeout=600, dots=True,
         debug_log = DebugLogBuffer(log)
         dots = False
     else:
-        debug_log = lambda c: None
+        def debug_log(c):
+            return None
 
     if WIN:
         start_time = [time.time()]
@@ -592,7 +455,8 @@ def check_output(args, valid_return_codes=(0,), timeout=600, dots=True,
         all_threads = [stdout_reader]
 
         if not redirect_stderr:
-            stderr_reader = threading.Thread(target=stream_reader, args=(proc.stderr, stderr_chunks))
+            stderr_reader = threading.Thread(target=stream_reader,
+                                             args=(proc.stderr, stderr_chunks))
             stderr_reader.daemon = True
             stderr_reader.start()
             all_threads.append(stderr_reader)
@@ -658,9 +522,7 @@ def check_output(args, valid_return_codes=(0,), timeout=600, dots=True,
                 signal.signal(signal.SIGTSTP, sig_forward)
                 signal.signal(signal.SIGCONT, sig_forward)
 
-            fds = {
-                proc.stdout.fileno(): stdout_chunks
-                }
+            fds = {proc.stdout.fileno(): stdout_chunks}
             if not redirect_stderr:
                 fds[proc.stderr.fileno()] = stderr_chunks
 
@@ -730,7 +592,9 @@ def check_output(args, valid_return_codes=(0,), timeout=600, dots=True,
 
     # Flush and disconnect debug log, if any
     debug_log(None)
-    debug_log = lambda c: None
+
+    def debug_log(c):
+        return None
 
     stdout = b''.join(stdout_chunks)
     stderr = b''.join(stderr_chunks)
@@ -744,7 +608,7 @@ def check_output(args, valid_return_codes=(0,), timeout=600, dots=True,
         retcode = proc.returncode
 
     if valid_return_codes is not None and retcode not in valid_return_codes:
-        header = 'Error running {0} (exit status {1})'.format(' '.join(args), retcode)
+        header = f"Error running {' '.join(args)} (exit status {retcode})"
         if display_error:
             if log.is_debug_enabled():
                 # Output was already printed
@@ -778,10 +642,7 @@ def is_main_thread():
     """
     Return True if the current thread is the main thread.
     """
-    if sys.version_info[0] >= 3:
-        return threading.current_thread() == threading.main_thread()
-    else:
-        return isinstance(threading.current_thread(), threading._MainThread)
+    return threading.current_thread() == threading.main_thread()
 
 
 def write_json(path, data, api_version=None, compact=False):
@@ -811,9 +672,7 @@ def write_json(path, data, api_version=None, compact=False):
         data['version'] = api_version
 
     open_kwargs = {}
-    if sys.version_info[0] >= 3:
-        open_kwargs['encoding'] = 'utf-8'
-
+    open_kwargs['encoding'] = 'utf-8'
     with long_path_open(path, 'w', **open_kwargs) as fd:
         if not compact:
             json.dump(data, fd, indent=4, sort_keys=True)
@@ -842,42 +701,38 @@ def load_json(path, api_version=None, js_comments=False):
     path = os.path.abspath(path)
 
     open_kwargs = {}
-    if sys.version_info[0] >= 3:
-        open_kwargs['encoding'] = 'utf-8'
-
+    open_kwargs['encoding'] = 'utf-8'
     with long_path_open(path, 'r', **open_kwargs) as fd:
         content = fd.read()
 
     if js_comments:
-        content = minify_json.json_minify(content)
-        content = content.replace(",]", "]")
-        content = content.replace(",}", "}")
-
-    try:
-        d = json.loads(content)
-    except ValueError as e:
-        raise UserError(
-            "Error parsing JSON in file '{0}': {1}".format(
-                path, six.text_type(e)))
+        # strips comments out
+        data = json5.loads(content)
+    else:
+        try:
+            data = json.loads(content)
+        except ValueError as err:
+            raise UserError(
+                f"Error parsing JSON in file '{path}': {str(err)}")
 
     if api_version is not None:
-        if 'version' in d:
-            if d['version'] < api_version:
+        if 'version' in data:
+            if data['version'] < api_version:
                 raise UserError(
                     "{0} is stored in an old file format.  Run "
                     "`asv update` to update it.".format(path))
-            elif d['version'] > api_version:
+            elif data['version'] > api_version:
                 raise UserError(
                     "{0} is stored in a format that is newer than "
                     "what this version of asv understands.  Update "
                     "asv to use this file.".format(path))
 
-            del d['version']
+            del data['version']
         else:
             raise UserError(
-                "No version specified in {0}.".format(path))
+                f"No version specified in {path}.")
 
-    return d
+    return data
 
 
 def update_json(cls, path, api_version, compact=False):
@@ -902,11 +757,11 @@ def update_json(cls, path, api_version, compact=False):
     d = load_json(path)
     if 'version' not in d:
         raise UserError(
-            "No version specified in {0}.".format(path))
+            f"No version specified in {path}.")
 
     if d['version'] < api_version:
-        for x in six.moves.xrange(d['version'] + 1, api_version + 1):
-            d = getattr(cls, 'update_to_{0}'.format(x), lambda x: x)(d)
+        for x in range(d['version'] + 1, api_version + 1):
+            d = getattr(cls, f'update_to_{x}', lambda x: x)(d)
         write_json(path, d, api_version, compact=compact)
     elif d['version'] > api_version:
         raise UserError(
@@ -1000,7 +855,7 @@ def get_cpu_info():
             from win32com.client import GetObject
             cimv = GetObject(r"winmgmts:root\cimv2")
             return cimv.ExecQuery("Select Name from Win32_Processor")[0].name
-        except:
+        except Exception:
             pass
     return ''
 
@@ -1027,44 +882,6 @@ def get_memsize():
     return ''
 
 
-def _get_terminal_size_fallback():
-    """
-    Returns a tuple (height, width) containing the height and width of
-    the terminal.  Fallback for when sys.get_terminal_size() doesn't
-    exist or fails.
-    """
-    try:
-        # Unix-specific code
-        import fcntl
-        import termios
-        s = struct.pack(str("HHHH"), 0, 0, 0, 0)
-        x = fcntl.ioctl(sys.stdout, termios.TIOCGWINSZ, s)
-        (lines, width, xpixels, ypixels) = struct.unpack(str("HHHH"), x)
-        if lines > 12:
-            lines -= 6
-        if width > 10:
-            width -= 1
-        return (lines, width)
-    except:
-        # Fall back on environment variables, or if not set, (25, 80)
-        try:
-            return (int(os.environ.get('LINES')),
-                    int(os.environ.get('COLUMNS')))
-        except TypeError:
-            return 25, 80
-
-
-def get_terminal_width():
-    """
-    Return the terminal width, or an estimate thereof.
-    """
-    try:
-        # Python 3.3 and higher: this works under Windows and Unix
-        return os.get_terminal_size().columns
-    except (AttributeError, OSError):
-        return _get_terminal_size_fallback()[1]
-
-
 def format_text_table(rows, num_headers=0,
                       top_header_span_start=0,
                       top_header_text=None):
@@ -1083,13 +900,13 @@ def format_text_table(rows, num_headers=0,
     """
 
     # Format content
-    text_rows = [["{0}".format(item).replace("\n", " ") for item in row]
+    text_rows = [[f"{item}".replace("\n", " ") for item in row]
                  for row in rows]
 
     # Ensure same number of items on all rows
     num_items = max(len(row) for row in text_rows)
     for row in text_rows:
-        row.extend(['']*(num_items - len(row)))
+        row.extend([''] * (num_items - len(row)))
 
     # Determine widths
     col_widths = [max(len(row[j]) for row in text_rows) + 2
@@ -1102,14 +919,14 @@ def format_text_table(rows, num_headers=0,
     # Generate result
     headers = [" ".join(row) for row in text_rows[:num_headers]]
     content = [" ".join(row) for row in text_rows[num_headers:]]
-    separator = " ".join("-"*w for w in col_widths)
+    separator = " ".join("-" * w for w in col_widths)
 
     result = []
     if top_header_text is not None:
-        left_span = "-".join("-"*w for w in col_widths[:top_header_span_start])
-        right_span = "-".join("-"*w for w in col_widths[top_header_span_start:])
+        left_span = "-".join("-" * w for w in col_widths[:top_header_span_start])
+        right_span = "-".join("-" * w for w in col_widths[top_header_span_start:])
         if left_span and right_span:
-            result += ["--" + " " * (len(left_span)-1) + top_header_text.center(len(right_span))]
+            result += ["--" + " " * (len(left_span) - 1) + top_header_text.center(len(right_span))]
             result += [" ".join([left_span, right_span])]
         else:
             result += [top_header_text.center(len(separator))]
@@ -1126,10 +943,13 @@ def format_text_table(rows, num_headers=0,
 
 
 def _datetime_to_timestamp(dt, divisor):
-    delta = dt - datetime.datetime(1970, 1, 1)
+    delta = dt - datetime.datetime(
+        1970, 1, 1,
+        tzinfo = datetime.timezone.utc
+    )
     microseconds = (delta.days * 86400 + delta.seconds) * 10**6 + delta.microseconds
     value, remainder = divmod(microseconds, divisor)
-    if remainder >= divisor//2:
+    if remainder >= divisor // 2:
         value += 1
     return value
 
@@ -1190,7 +1010,7 @@ def geom_mean_na(values):
     """
     values = [x for x in values if not is_na(x)]
     if values:
-        exponent = 1/len(values)
+        exponent = 1 / len(values)
         prod = 1.0
         acc = 0
         for x in values:
@@ -1201,14 +1021,10 @@ def geom_mean_na(values):
         return None
 
 
-def ceildiv(numerator, denominator):
-    """Ceiling division"""
-    return -((-numerator)//denominator)
-
-
 if not WIN:
     long_path_open = open
     long_path_rmtree = shutil.rmtree
+
     def long_path(path):
         return path
 else:
@@ -1230,7 +1046,7 @@ else:
                 pass
 
         # Reraise original error
-        six.reraise(*exc_info)
+        raise
 
     def long_path_open(filename, *a, **kw):
         return open(long_path(filename), *a, **kw)
@@ -1253,7 +1069,7 @@ def sanitize_filename(filename):
 
     The implementation needs to match www/asv.js:escape_graph_parameter
     """
-    if not isinstance(filename, six.text_type):
+    if not isinstance(filename, str):
         filename = filename.decode(sys.getfilesystemencoding())
 
     # ntfs & ext3
@@ -1272,11 +1088,8 @@ def sanitize_filename(filename):
 
 def namedtuple_with_doc(name, slots, doc):
     cls = collections.namedtuple(name, slots)
-    if sys.version_info[0] >= 3:
-        cls.__doc__ = doc
-        return cls
-    else:
-        return type(str(name), (cls,), {'__doc__': doc})
+    cls.__doc__ = doc
+    return cls
 
 
 def recvall(sock, size):
@@ -1368,7 +1181,7 @@ def interpolate_command(command, variables):
                     return_codes_set = True
                     del result[0]
                     continue
-                except ValueError as exc:
+                except ValueError:
                     pass
 
             raise UserError("Configuration error: invalid return-code specification "
@@ -1398,7 +1211,7 @@ def truncate_float_list(item, digits=5):
     representation.
     """
     if isinstance(item, float):
-        fmt = '{{:.{}e}}'.format(digits - 1)
+        fmt = f'{{:.{digits - 1}e}}'
         return float(fmt.format(item))
     elif isinstance(item, list):
         return [truncate_float_list(x, digits) for x in item]
@@ -1445,3 +1258,159 @@ except ImportError:
         # use single quotes, and put single quotes into double quotes
         # the string $'b is then quoted as '$'"'"'b'
         return "'" + s.replace("'", "'\"'\"'") + "'"
+
+
+def git_default_branch():
+    try:
+        # Local name gets precedence
+        default_branch = check_output(
+            [which('git'), 'config', 'init.defaultBranch'],
+            display_error=False).strip()
+    except ProcessError:
+        # Check global
+        try:
+            default_branch = check_output(
+                [which('git'), 'config', '--global', 'init.defaultBranch'],
+                display_error=False).strip()
+        except ProcessError:
+            # Check system
+            try:
+                default_branch = check_output(
+                    [which('git'), 'config', '--system', 'init.defaultBranch'],
+                    display_error=False).strip()
+            except ProcessError:
+                # Default to master when global and system are not set
+                default_branch = 'master'
+    return default_branch
+
+
+def search_channels(cli_path, pkg, version):
+    try:
+        result = subprocess.run([cli_path, "search",
+                                 f"{pkg}=={version}"],
+                                capture_output=True,
+                                text=True,
+                                check=False)
+    except subprocess.CalledProcessError as e:
+        print(f"Error searching for {pkg} {version}, got:\n {e}",
+              file=sys.stderr)
+        return False
+    if f"No match found for: {pkg}=={version}." in result.stdout:
+        return False
+    # Worked!
+    return True
+
+class ParsedPipDeclaration:
+    def __init__(self, declaration):
+        self.pkgname = None
+        self.specification = None
+        self.flags = []
+        self.is_editable = False
+        self.path = None
+
+        self._parse_declaration(declaration)
+
+        if not self.pkgname and not self.path:
+            raise ValueError("Either a valid package name"
+                             " or a path must be present in the declaration.")
+
+    def _parse_declaration(self, declaration):
+        # Match flags with values
+        flag_with_value_pattern = (
+            r'(--[\w-]+='      # Match the flag name
+            r'\"[^\"]+\")'    # Match the value in double quotes
+        )
+        flag_values = re.findall(flag_with_value_pattern, declaration)
+        for flag_value in flag_values:
+            self.flags.append(flag_value)
+            declaration = declaration.replace(flag_value, '', 1)
+
+        # Match git URLs
+        git_url_pattern = (
+            r'(git\+https:\/\/[a-zA-Z0-9-_\/.]+)' # match the git URL
+            r'(?:#egg=([a-zA-Z0-9-_]+))?' # optional egg fragment
+        )
+        git_url_match = re.search(git_url_pattern, declaration)
+
+        # If there's a git URL match, remove it from the declaration
+        if git_url_match:
+            self.path = git_url_match.group(1)
+            if git_url_match.group(2):
+                self.pkgname = git_url_match.group(2)
+            declaration = declaration.replace(git_url_match.group(0), '', 1)
+
+        # Match local paths
+        local_pattern = (
+            r'(\.\/[a-zA-Z0-9-_]+\/?'          # Relative path starting with ./
+            r'|\.\.\/[a-zA-Z0-9-_]+\/?'        # Relative path starting with ../
+            r'|\/\w+\/?)'                      # Absolute path
+        )
+        local_match = re.search(local_pattern, declaration)
+
+        # If there's a local path match, remove it from the declaration
+        if local_match:
+            self.path = local_match.group(1)
+            declaration = declaration.replace(local_match.group(0), '', 1)
+
+        # Match flags
+        flags_pattern = (
+            r'(?:^|\s)'                        # Match start or whitespace
+            r'(-[a-zA-Z]|'                     # Single-letter flags
+            r'--\w+(?:-\w+)*)'                 # Double-dash flags
+        )
+        flags = re.findall(flags_pattern, declaration)
+        if flags:
+            self.flags.extend(flags)
+            if "-e" in self.flags:
+                self.is_editable = True
+
+        # Remove matched flags from declaration
+        for flag in self.flags:
+            declaration = declaration.replace(flag, '', 1)
+
+        # Match package details
+        pkg_pattern = (
+            r'(?P<name>[a-zA-Z0-9-_]+)'               # Name
+            r'('                                      # Start group for version specification(s)
+            r'((?P<specifier>[<>!=~]{1,2})'           # Version specifier
+            r'(?P<version>[0-9.a-zA-Z_-]+))'          # Version
+            r'((?P<multi_spec>,[<>!=~]{1,2}[0-9.a-zA-Z_-]+)*)?' # Multiple version specifications
+            r')?' # End optional group for version specification(s)
+        )
+        pkg_match = re.search(pkg_pattern, declaration)
+
+        # Populate attributes based on package details matches
+        if pkg_match:
+            self.pkgname = pkg_match.group("name")
+            specifier = pkg_match.group("specifier")
+            version = pkg_match.group("version")
+            multi_spec = pkg_match.group("multi_spec")
+
+            # If a specifier is present, prioritize it and consume version(s) greedily
+            if specifier:
+                self.specification = f"{specifier}{version}{multi_spec or ''}"
+            else:
+                declaration = declaration.replace(pkg_match.group("name"), '', 1)
+                version_match = re.search(r'(?P<version>\d+(\.\d+)*([a-zA-Z0-9]+)?)', declaration)
+                if version_match:
+                    self.specification = f"=={version_match.group(0)}"
+
+def construct_pip_call(pip_caller, parsed_declaration: ParsedPipDeclaration):
+    pargs = ['install', '-v', '--upgrade']
+
+    if parsed_declaration.flags:
+        pargs += parsed_declaration.flags
+    if parsed_declaration.path:
+        pargs.append(parsed_declaration.path)
+    elif parsed_declaration.pkgname:
+        if parsed_declaration.specification:
+            pargs.append(f"{parsed_declaration.pkgname}{parsed_declaration.specification}")
+        else:
+            pargs.append(parsed_declaration.pkgname)
+
+    return functools.partial(pip_caller, pargs)
+
+if hasattr(sys, 'pypy_version_info'):
+    ON_PYPY = True
+else:
+    ON_PYPY = False
