@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function)
-
-
 class Suite:
 
     def named_method(self):
