@@ -1,23 +1,14 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-from os.path import dirname, join
 import shutil
+from os.path import join
 
-import six
-
-from asv import config
-from asv import results
+from asv import config, results
 
 from . import tools
-from .tools import example_results
 
 
 def test_rm(tmpdir, example_results):
-    tmpdir = six.text_type(tmpdir)
+    tmpdir = str(tmpdir)
 
     shutil.copytree(
         example_results,
@@ -26,7 +17,7 @@ def test_rm(tmpdir, example_results):
     conf = config.Config.from_json({
         'results_dir': join(tmpdir, 'example_results'),
         'repo': "### IGNORED, BUT REQUIRED ###"
-        })
+    })
 
     tools.run_asv_with_conf(conf, 'rm', '-y', 'benchmark=time_quantity*')
 
@@ -34,9 +25,9 @@ def test_rm(tmpdir, example_results):
     for result in results_a:
         for key in result.get_all_result_keys():
             assert not key.startswith('time_quantity')
-        for key in six.iterkeys(result.started_at):
+        for key in result.started_at.keys():
             assert not key.startswith('time_quantity')
-        for key in six.iterkeys(result.duration):
+        for key in result.duration.keys():
             assert not key.startswith('time_quantity')
 
     tools.run_asv_with_conf(conf, 'rm', '-y', 'commit_hash=05d283b9')
