@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import os
 import shutil
 import hashlib
@@ -11,7 +7,6 @@ import hashlib
 from asv import util
 
 from . import tools
-from .test_publish import generate_result_dir
 
 
 def test_update_simple(monkeypatch, generate_result_dir):
@@ -36,10 +31,8 @@ def test_update_simple(monkeypatch, generate_result_dir):
 
     result_fns = [fn for fn in sorted(os.listdir(machine_dir))
                   if fn != 'machine.json']
-    long_result_fn = 'abbacaca-' + 'a'*128 + '.json'
-    hash_result_fn = ('abbacaca-env-'
-                      + hashlib.md5(b'a'*128).hexdigest()
-                      + '.json')
+    long_result_fn = 'abbacaca-' + 'a' * 128 + '.json'
+    hash_result_fn = ('abbacaca-env-' + hashlib.md5(b'a' * 128).hexdigest() + '.json')
 
     shutil.copyfile(os.path.join(machine_dir, result_fns[0]),
                     os.path.join(machine_dir, long_result_fn))
