@@ -1,11 +1,4 @@
-# -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import io
-import locale
 import os
 import sys
 import shutil
@@ -13,15 +6,12 @@ import pickle
 import multiprocessing
 import threading
 import traceback
-import time
 import datetime
-import six
+
 import pytest
 
-from asv import console
 from asv import util
 
-
 WIN = (os.name == 'nt')
 
 
@@ -64,7 +54,7 @@ def test_parallelfailure():
     pool = multiprocessing.Pool(4)
     try:
         pool.map(_multiprocessing_raise_processerror, range(10))
-    except util.ParallelFailure as exc:
+    except util.ParallelFailure:
         pass
     finally:
         pool.close()
@@ -79,7 +69,7 @@ def test_parallelfailure():
         finally:
             pool.close()
         assert False
-    except util.UserError as exc:
+    except util.UserError:
         # OK
         pass
 
@@ -114,7 +104,7 @@ def test_which_path(tmpdir):
         util.which('asv_test_exe_4321.bat', paths=[dirname])
 
         # Check non-existent files
-        with pytest.raises(IOError):
+        with pytest.raises(OSError):
             util.which('nonexistent.exe', paths=[dirname])
     finally:
         os.environ['PATH'] = old_path
@@ -128,7 +118,7 @@ def test_write_load_json(tmpdir):
     }
     orig_data = dict(data)
 
-    filename = os.path.join(six.text_type(tmpdir), 'test.json')
+    filename = os.path.join(str(tmpdir), 'test.json')
 
     util.write_json(filename, data)
     data2 = util.load_json(filename)
@@ -207,7 +197,7 @@ def test_human_time():
         ("1.12s", 1.123),
         ("1.13s", 1.126),
         ("1.00m", 60),
-        ("2.00h", 3600*2),
+        ("2.00h", 3600 * 2),
         ("0s", 0),
         ("n/a", float("nan")),
 
@@ -260,9 +250,9 @@ def test_human_file_size():
 def test_parse_human_time():
     items = [
         # (value, expected)
-        ("1", 60*60*24),
-        ("1h", 60*60),
-        ("1w", 60*60*24*7),
+        ("1", 60 * 60 * 24),
+        ("1h", 60 * 60),
+        ("1w", 60 * 60 * 24 * 7),
     ]
 
     for value, expected in items:
@@ -362,19 +352,28 @@ def test_datetime_to_js_timestamp():
     tss = [0, 0.5, -0.5, 12345.6789, -12345.6789,
            1535910708.7767508]
     for ts in tss:
-        t = datetime.datetime.utcfromtimestamp(ts)
+        t = datetime.datetime.fromtimestamp(ts, tz=datetime.timezone.utc)
         ts2 = util.datetime_to_js_timestamp(t)
         assert abs(ts * 1000 - ts2) <= 0.5
 
     # Check sub-second precision
     ms = 50
-    ts = datetime.datetime(1970, 1, 1, 0, 0, 0, 1000*ms)
+    ts = datetime.datetime(
+        1970, 1, 1, 0, 0, 0, 1000 * ms,
+        tzinfo = datetime.timezone.utc
+    )
     assert util.datetime_to_js_timestamp(ts) == ms
 
     # Check rounding
-    ts = datetime.datetime(1970, 1, 1, 0, 0, 0, 500)
+    ts = datetime.datetime(
+        1970, 1, 1, 0, 0, 0, 500,
+        tzinfo = datetime.timezone.utc
+    )
     assert util.datetime_to_js_timestamp(ts) == 1
-    ts = datetime.datetime(1970, 1, 1, 0, 0, 0, 499)
+    ts = datetime.datetime(
+        1970, 1, 1, 0, 0, 0, 499,
+        tzinfo = datetime.timezone.utc
+    )
     assert util.datetime_to_js_timestamp(ts) == 0
 
 
@@ -382,14 +381,21 @@ def test_datetime_to_timestamp():
     tss = [0, 0.5, -0.5, 12345.6789, -12345.6789,
            1535910708.7767508]
     for ts in tss:
-        t = datetime.datetime.utcfromtimestamp(ts)
+        t = datetime.datetime.fromtimestamp(ts,
+                                            tz=datetime.timezone.utc)
         ts2 = util.datetime_to_timestamp(t)
         assert abs(ts - ts2) <= 0.5
 
     # Check rounding
-    ts = datetime.datetime(1970, 1, 1, 0, 0, 0, 500000)
+    ts = datetime.datetime(
+        1970, 1, 1, 0, 0, 0, 500000,
+        tzinfo = datetime.timezone.utc
+    )
     assert util.datetime_to_timestamp(ts) == 1
-    ts = datetime.datetime(1970, 1, 1, 0, 0, 0, 500000 - 1)
+    ts = datetime.datetime(
+        1970, 1, 1, 0, 0, 0, 500000 - 1,
+        tzinfo = datetime.timezone.utc
+    )
     assert util.datetime_to_timestamp(ts) == 0
 
 
@@ -402,5 +408,127 @@ def test_check_output_exit_code(capsys):
 
 def test_geom_mean_na():
     for x in [[1, 2, -3], [1, 2, 3], [3, 1, 3, None, None]]:
-        expected = abs(x[0]*x[1]*x[2])**(1/3)
+        expected = abs(x[0] * x[1] * x[2])**(1 / 3)
         assert abs(util.geom_mean_na(x) - expected) < 1e-10
+
+def mock_pip_caller(args):
+    return args
+
+@pytest.mark.parametrize("declaration, expected", [
+    # Basic package name
+    ("numpy", {
+        'pkgname': 'numpy', 'specification': None,
+        'flags': [], 'is_editable': False, 'path': None
+    }),
+
+    # Version with '=='
+    ("numpy==1.20.0", {
+        'pkgname': 'numpy', 'specification': '==1.20.0',
+        'flags': [], 'is_editable': False, 'path': None
+    }),
+
+    # Other specifiers
+    ("numpy>=1.20.0", {
+        'pkgname': 'numpy', 'specification': '>=1.20.0',
+        'flags': [], 'is_editable': False, 'path': None
+    }),
+
+    ("numpy<=1.20.0", {
+        'pkgname': 'numpy', 'specification': '<=1.20.0',
+        'flags': [], 'is_editable': False, 'path': None
+    }),
+
+    # Complex version specifiers
+    ("numpy>=1.15,<2.0", {
+        'pkgname': 'numpy', 'specification': '>=1.15,<2.0',
+        'flags': [], 'is_editable': False, 'path': None
+    }),
+
+    # Flags
+    ("--no-cache-dir numpy", {
+        'pkgname': 'numpy', 'specification': None,
+        'flags': ['--no-cache-dir'], 'is_editable': False, 'path': None
+    }),
+
+    # Editable installations
+    ("-e ./numpy-dir", {
+        'pkgname': None, 'specification': None,
+        'flags': ['-e'], 'is_editable': True, 'path': './numpy-dir'
+    }),
+
+    # Local paths without -e
+    ("./numpy-dir", {
+        'pkgname': None, 'specification': None,
+        'flags': [], 'is_editable': False, 'path': './numpy-dir'
+    }),
+
+    # Package with dash
+    ("my-package", {
+        'pkgname': 'my-package', 'specification': None,
+        'flags': [], 'is_editable': False, 'path': None
+    }),
+
+    # More complex declarations
+    ("--no-cache-dir -e ./numpy-dir", {
+        'pkgname': None, 'specification': None,
+        'flags': ['--no-cache-dir', '-e'], 'is_editable': True, 'path': './numpy-dir'
+    }),
+
+    # Direct git installations
+    ("git+https://github.com/user/repo.git", {
+        'pkgname': None, 'specification': None,
+        'flags': [], 'is_editable': False, 'path': 'git+https://github.com/user/repo.git'
+    }),
+    # Editable git installations with #egg= suffix
+    ("-e git+https://github.com/user/repo.git#egg=repo", {
+        'pkgname': 'repo', 'specification': None,
+        'flags': ['-e'], 'is_editable': True, 'path': 'git+https://github.com/user/repo.git'
+    }),
+    # Flags with values
+    ("--install-option=\"--prefix=/my/path\" numpy", {
+        'pkgname': 'numpy', 'specification': None,
+        'flags': ['--install-option=\"--prefix=/my/path\"'], 'is_editable': False, 'path': None
+    }),
+])
+def test_parsed_pip_declaration(declaration, expected):
+    parsed = util.ParsedPipDeclaration(declaration)
+    for key, value in expected.items():
+        assert getattr(parsed, key) == value, \
+            f"Expected {key} to be {value}, but got {getattr(parsed, key)}"
+
+# Mock pip_caller
+def pip_caller(args):
+    return args
+
+@pytest.mark.parametrize(
+    "declaration, expected_result",
+    [
+        # Test with a simple package name
+        ("numpy", ["install", "-v", "--upgrade", "numpy"]),
+
+        # Test with a package and version specification
+        ("numpy==1.18.5", ["install", "-v", "--upgrade", "numpy==1.18.5"]),
+
+        # Test with a git URL
+        ("git+https://github.com/numpy/numpy.git#egg=numpy",
+         ["install", "-v", "--upgrade", "git+https://github.com/numpy/numpy.git"]),
+
+        # Test with a local path
+        ("./localpackage/", ["install", "-v", "--upgrade", "./localpackage/"]),
+
+        # Test with flags
+        ("numpy --install-option=\"--prefix=/my/path\"",
+         ["install", "-v", "--upgrade", "--install-option=\"--prefix=/my/path\"", "numpy"]),
+
+        # Test case for multiple version specifiers
+        ("numpy>=1.18.5,<=1.20.0", ["install", "-v", "--upgrade", "numpy>=1.18.5,<=1.20.0"]),
+
+        # Test cases for version without specifier
+        ("numpy 1.23", ["install", "-v", "--upgrade", "numpy==1.23"]),
+        ("fakepy_1 0.14", ["install", "-v", "--upgrade", "fakepy_1==0.14"]),
+    ]
+)
+def test_construct_pip_call(declaration, expected_result):
+    parsed_declaration = util.ParsedPipDeclaration(declaration)
+    result = util.construct_pip_call(pip_caller, parsed_declaration)
+    assert result() == expected_result
